require_relative '../spec_helper'
require './lib/studio_game/player'
require './lib/studio_game/treasure_trove'

module StudioGame
  describe Player do
    before do
      $stdout = StringIO.new
      @start_health = 150
      @player = Player.new("larry", @start_health)
    end
    it "can create a player from a CSV string" do
      player = Player.from_csv("bill,20")
      player.name.should == "Bill"
      player.health.should == 20
    end
    it "has a capitalized name" do
      @player.name.should == "Larry"
    end
    it "has an initial health" do
      @player.health.should == @start_health
    end
    it "has a string representation" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))

      @player.to_s.should == "I'm Larry with health = 150, points = 100, and score = 250."
    end
    it "computes a score as the sum of the health and points" do
      @player.score.should == @player.health + @player.points
    end
    it "increases health by 15 when w00ted" do
      @player.w00t
      @player.health.should == @start_health + 15
    end
    it "decreases health by 10 when blammed" do
      @player.blam
      @player.health.should == @start_health - 10
    end
    it "yields each found treasure and its total points" do
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))

      yielded = []
      @player.each_found_treasure do |treasure|
        yielded << treasure
      end

      yielded.should == [
        Treasure.new(:skillet, 200),
        Treasure.new(:hammer, 50),
        Treasure.new(:bottle, 25)
     ]
    end

  	context "player health greater than 100" do
  		before do
  			@player = Player.new("larry",150)
  		end
  	  it "is strong" do
  			@player.should be_strong
  	    @player.strong?.should == true
        @player.strong?.should be_true
        # @player.strong?.should be_truthy
  			expect(@player.strong?).to eq(true)
    	end
    end

  	context "a player with health less than or equal to 100" do
  		before do
  			@player = Player.new('larry', 100)
  		end
  		it "is weak" do
  			@player.should_not be_strong
  		end
  	end

  	context "a collection of players" do
  		before do
  			@player1 = Player.new('moe', 100)
  			@player2 = Player.new('larry', 200)
  			@player3 = Player.new('curly', 300)

  			@players = [@player1, @player2, @player3]
  		end
  		it "sorts by decreasing score" do
  			@players.sort.should == [@player3, @player2, @player1]
  		end
  	end

  	it "computes points as the sum of all treasure points" do
      @player.points.should == 0
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.points.should == 50
      @player.found_treasure(Treasure.new(:crowbar, 400))
      @player.points.should == 450
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.points.should == 500
    end

  	it "computes a score as the sum of its health and points" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.score.should == 250
    end

  end
end
